/**
 * The SaveGameManager allows for the saving and loading of Kobold Adventure.
 * It also allows for easy enumeration over the various saves.
 */
function SaveManager() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var autoSaveSlot = "autosave";
    var newGameSlot = "new";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Fetches data from localstorage at the specified slot,
     * converts the data into an object,
     * and returns that object.
     * @param {int} slot The slot to fetch data from.
     * @returns {object} The JSON data contained in that slot, parsed to an object.
     */
    function readSlot(slot) {
        var json = localStorage[slot];
        return JSON.parse(json);
    }

    /**
     * Checks whether the slot is occupied or not.
     * @param {int} slot The slot to check.
     * @returns {boolean} True if the slot is occupied, false otherwise.
     */
    function slotOccupied(slot) {
        return localStorage.getItem(slot) !== null;
    }

    /**
     * Builds a SaveMetaData object from the save in the passed slot.
     * @param {string} slot The slot to read from.
     * @returns {SaveMetaData} The constructed SaveMetaData object.
     */
    function metaDataFromSlot(slot) {
        var obj = readSlot(slot);
        return new SaveMetaData(slot, obj.scene, obj.sceneName, obj.comment, obj.kobold.name, obj.kobold.body.gender, obj.time);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.maxAmountOfSaves = 99;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Autosaves the game.
     * @param {bool} noThrobber If thruthy, no saving throbber will be shown. May be omitted.
     */
    this.autoSave = function (noThrobber) {
        that.save(autoSaveSlot, "Autosave", noThrobber);
    };

    /**
     * Saves the current state to the new game slot.
     */
    this.newGameSave = function () {
        that.save(newGameSlot, "New game", true);
    };

    /**
     * Checks if an autosave exists.
     * @returns {boolean} True if it does, false otherwise.
     */
    this.hasAutoSave = function () {
        return slotOccupied(autoSaveSlot);
    };

    /**
     * Checks if a new game save exists.
     * @returns {boolean} True if it does, false otherwise.
     */
    this.hasNewGameSave = function () {
        return slotOccupied(newGameSlot);
    };

    /**
     * Gets the meta information of the save in the autosave slot.
     */
    this.getAutoSaveMeta = function () {
        return metaDataFromSlot(autoSaveSlot);
    };

    /**
     * Gets the meta information of the save in the new game slot.
     */
    this.getNewGameSaveMeta = function () {
        return metaDataFromSlot(newGameSlot);
    };

    /**
     * Loads the game from the auto save slot.
     */
    this.loadAutoSave = function () {
        that.load(autoSaveSlot);
    };

    /**
     * Loads the game from the start save slot.
     */
    this.loadNewGame = function () {
        that.load(newGameSlot);
    };

    /**
     * Saves the game. 
     * @param {int} slot The slot to save to.
     * @param {string} comment A comment to store alongside the save. May be empty.
     * @param {bool} noThrobber If thruthy, no saving throbber will be shown. May be omitted.
     */
    this.save = function (slot, comment, noThrobber) {
        if (!noThrobber) {
            global.ui.throbbers.save.showAndHide();
        }
        if (!comment) {
            comment = "";
        }
        var toSave = new KoboldAdventureSave(global.kobold, global.currentScene, global.currentSceneName, global.currentSceneFolder, comment, Date.now());
        localStorage.setItem(slot, toSave.toJson());
    };

    /**
     * Loads the game from a saveslot.
     * Sets the global kobold object, 
     * fades out the current content 
     * and then loads the new scene.
     * @param {int} slot The slot to load from.
     */
    this.load = function (slot) {
        var obj = readSlot(slot);
        var kobold = new Character();
        kobold.load(obj.kobold);
        global.kobold = kobold;
        global.ui.fadeOutContent(function () {
            global.ui.tabs.scene.activate();
            global.sceneLoader.loadScene(obj.scene, obj.folder);
        });
    };

    /**
     * Deletes the save at the specified slot.
     * @param {int} slot The slot to delete.
     */
    this.delete = function (slot) {
        localStorage.removeItem(slot);
    };

    /**
     * Fetches and returns the Meta object for the save at the passed slot.
     * @param {string} slot The slot for which to fetch meta data.
     * @returns {SaveMetaData} The fetched SaveMetaData object.
     */
    this.getMeta = function (slot) {
        return metaDataFromSlot(slot);
    };

    /**
     * Looks up and returns the id of the first open save slot.
     * Returns -1 if no save slot is available.
     * @returns {int} The id of the first open save slot. -1 if none are available.
     */
    this.getFirstOpenSaveSlot = function () {
        var i = 0;

        while (slotOccupied(i)) {
            i++;
        }

        if (i >= that.maxAmountOfSaves) {
            return -1;
        }

        return i;
    };

    /**
     * Fetches a summation list of save slots.
     * The list will contain one SaveMetaData object entry per save in use.
     * The list may be empty.
     * Does not include information about the autosave or the new game save.
     * @returns {array} An array of SaveMetaData objects.
     */
    this.saveList = function () {
        var toReturn = [];

        for (var i = 0; i < that.maxAmountOfSaves; i++) {
            if (slotOccupied(i)) {
                toReturn.push(metaDataFromSlot(i));
            }
        }

        return toReturn;
    };

    /**
     * Returns how many saves there are currently present, apart from the
     * autosave and the starting save. The same as the length of the save list.
     * @returns {int} How many saves there currently are.
     */
    this.getAmountOfSaves = function () {
        var toReturn = 0;

        for (var i = 0; i < that.maxAmountOfSaves; i++) {
            if (slotOccupied(i)) {
                toReturn++;
            }
        }

        return toReturn;
    };
}
