/**
 * Listens to changes in a single setting, and passes the new value on to all registered listeners.
 * @param {SettingChangeMasterListener} masterListener The MasterListener providing this SubListener with changed settings objects.
 * @param {string} setting The key for the setting itself.
 * @param {function} transform Transforms the changed setting before passing it to the listeners.
 *                             Gets the value as a parameter. Must return the transformed value. May be omitted.
 */
function SettingSubListener(masterListener, setting, transform) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // Array of listener functions.
    var listeners = [];

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Gets registered to the masterListener.
     * Listen to changes in this setting, and notifies the listeners if there are any changes.
     * @param {object} oldValue The old settings object.
     * @param {object} newValue The new settings object.
     */
    function listen(oldValue, newValue) {
        oldValue = oldValue || {};
        var oldSetting = oldValue[setting];
        var newSetting = newValue[setting];
        if (oldSetting === newSetting) {
            return;
        }
        if (transform) {
            newSetting = transform(newSetting);
        }
        for (var i = 0; i < listeners.length; i++) {
            listeners[i](newSetting);
        }
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    /**
     * Registers a listener.
     * The listener will be passed the changed value, potentially altered by the transformation function (if there is one).
     * @param {function} fun The function to unregister.
     */
    this.register = function (fun) {
        listeners.push(fun);
    };

    /**
     * Unregisters a listener.
     * @param {function} fun The function to unregister.
     */
    this.unregister = function (fun) {
        var index = listeners.indexOf(fun);
        if (index < 0 || index >= listeners.length) {
            return;
        }
        listeners.splice(index, 1);
    };

    // Register this listener.
    masterListener.register(listen);
}

