/**
 * This class contains functions specific to the Save tab.
 * Its init() function should be called immediately upon successful project load.
 * Not the best implementation. Can be improved in many ways.
 * @param {GenericTab} generic An optional GenericTab to prevent having to create multiple copies.
 */
function SaveTab(generic) {
    // PRIVATE MEMBERS \\       var private = "test";
    var gen = generic ? generic : new GenericTab();
    var id = "savetab";
    var title = "Save";
    var deleteButtonId = "deletebutton";
    var saveButtonId = "savebutton";
    var newCommentFieldId = "newcomment";
    var selected = "";
    var newSaveSlotId = "";
    var autoSaveSlotId = "";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * The onclick function for this tab.
     * Does nothing if the tab is already active.
     */
    function onClick() {
        if (gen.isActive(id)) {
            return;
        }
        gen.tabButtonTransition(id);
        global.sceneLoader.loadTabScene(id);
    }

    /**
     * Registers the onclick function for this tab.
     */
    function registerListeners() {
        $("#" + id).click(onClick);
    }

    /**
     * Builds and returns the new save slot HTML string.
     * @param {string} passedId If passed, will overwrite open slot lookup.
     * @returns {string} The new save slot HTML string.
     */
    function buildNewSaveSlot(passedId) {
        var saveId;
        if (passedId) {
            saveId = passedId;
        } else {
            saveId = global.saveManager.getFirstOpenSaveSlot();
        }
        if (saveId < 0) {
            return "";
        }

        newSaveSlotId = saveId;
        var slot = `<div class="${gen.smallSaveSlotClass}" id="${saveId}">
                        <img class="savegender" src="${global.kobold.body.getGenderIcon()}" alt="${global.kobold.body.getGenderName()}">
                        <div class="saverank">New save</div>    
                        <div class="savename"><span class="bold">${global.kobold.name}</span></div>
                        <div class="savescene">${global.currentSceneName}</div>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns the auto save slot HTML string.
     * @returns {string} The auto save slot HTML string.
     */
    function buildAutoSaveSlot() {
        var save = global.saveManager.getAutoSaveMeta();
        autoSaveSlotId = save.slot;
        return buildSmallSlotWithTime(save, "Autosave");
    }

    /**
     * Builds and returns a small save slot, with comment box omitted.
     * @param {object} save The save to create a small slot for. 
     * @param {int} rank The rank (= slot number) of the save.
     * @returns {string} The save slot.
     */
    function buildSmallSlotWithTime(save, rank) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.smallSaveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${rank}</div>
                        <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>        
                        <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns a generic save slot for loading purposes.
     * @param {object} save The save to create a slot for.
     * @returns {string} The save slot.
     */
    function buildSaveSlot(save) {
        var time = new Date(save.time);
        var slot = `<div class="${gen.saveSlotClass}" id="${save.slot}">
                        <img class="savegender" src="${GenderEnum.properties[save.gender].icon}" alt="${GenderEnum.properties[save.gender].name}">
                        <div class="saverank">${+save.slot + 1}</div>
                        <div class="savetimestamp">${gen.timeToTimePassed(time)}</div>
                        <div class="savename"><span class="bold">${save.name}</span></div>
                        <div class="savescene">${save.sceneName}</div>
                        <textarea rows="2" cols="60" class="savecomment" disabled>${save.comment}</textarea>
                    </div>`;
        return slot;
    }

    /**
     * Builds and returns the save list.
     * @param {array} saveMeta The list of save metadata. 
     * @returns {string} The save list.
     */
    function buildSaveList(saveMeta) {
        var saveList = buildNewSaveSlot();
        saveList += buildAutoSaveSlot();

        for (var i = 0; i < saveMeta.length; i++) {
            saveList += buildSaveSlot(saveMeta[i]);
        }

        return saveList;
    }

    /**
     * Builds and returns an additional finisher.
     */
    function extraFinisher() {
        return `<div class="finish"></div>`;
    }

    /**
     * Builds and returns the control bar, containing the save and delete buttons.
     */
    function buildControlBar() {
        return `<div class="savecontrolbar">
                    <button class="button saveleftbutton warning" id="${deleteButtonId}" disabled>Delete</button>
                    <textarea rows="2" cols="60" class="savecomment savenewcommentfield" placeholder="Comment. Old comment if left blank." id="${newCommentFieldId}"></textarea>
                    <button class="button saverightbutton" id="${saveButtonId}" disabled>Save</button>
                </div>`;
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var saveMeta = global.saveManager.saveList();
        var tab = gen.buildMaxSaveIndicator(saveMeta);
        tab += gen.buildTabHeader(title);
        tab += buildSaveList(saveMeta);
        tab += extraFinisher();
        tab += extraFinisher();
        tab += gen.buildTabFooter();
        tab += buildControlBar();
        return tab;
    }

    /**
     * Registers a listener for the new save slot.
     */
    function registerNewSaveListener() {
        if (newSaveSlotId === "") {
            return;
        }
        $("#" + newSaveSlotId).click(function () {
            gen.disable(deleteButtonId);
            gen.enable(saveButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Registers a listener for the autosave slot.
     */
    function registerAutoSaveListener() {
        $("#" + autoSaveSlotId).click(function () {
            gen.disable(deleteButtonId);
            gen.disable(saveButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Selects this element and unselects all others.
     * @param {DOMelement} element The element to select.
     */
    function selectSlot(element) {
        gen.removeSaveSlotSelected();
        element.addClass(gen.saveSlotSelectedClass);
        selected = element.attr('id');
    }

    /**
     * Registers listeners for the list of saveslots.
     * @param {string} selector The selector pointing to the list of saveslots.
     */
    function registerListListeners(selector) {
        $(selector).click(function () {
            gen.enable(deleteButtonId);
            gen.enable(saveButtonId);
            selectSlot($(this));
        });
    }

    /**
     * Registers a listener for the delete button.
     */
    function registerDeleteButtonListener() {
        $("#" + deleteButtonId).click(function () {
            if (confirm("Are you sure you want to delete this save?")) {
                gen.disable(deleteButtonId);
                gen.disable(saveButtonId);
                global.saveManager.delete(selected);
                gen.updateMaxSaveIndicator();
                var toRemove = $("#" + selected);
                toRemove.hide(400, function () {
                    toRemove.remove();
                });
                showNewSaveSlot(selected);
            }
        });
    }

    /**
     * This function hides the new save slot and disables both save and load
     * buttons (forcing a repick). Should be done when no more save slots are
     * available.
     */
    function hideNewSaveSlotAndDisableStuff() {
        var element = $("#" + newSaveSlotId);
        element.attr("id", "-1");
        element.hide(400);
        gen.disable(deleteButtonId);
        gen.disable(saveButtonId);
        selected = "";
        newSaveSlotId = "-1";
    }

    /**
     * Shows the new save slot and updates its id.
     * Does nothing if the save slot is visible.
     * Adds the save slot if necessary.
     * @param {string} newId The new id.
     */
    function showNewSaveSlot(newId) {
        if (newSaveSlotId === "") {
            var slot = buildNewSaveSlot(newId);
            $("#" + autoSaveSlotId).before(slot);
            $("#" + newId).hide(0);
            $("#" + newId).show(400);
            newSaveSlotId = newId;
            registerNewSaveListener();
        } else if (!($("#" + newSaveSlotId).is(":visible"))) {
            updateNewSaveSlot(newId);
            $("#" + newId).show(400);
        } else if (newId < newSaveSlotId) {
            updateNewSaveSlot(newId);
        }
    }

    /**
     * Updates the new save slot to a new id.
     * @param {string} newId The new id.
     */
    function updateNewSaveSlot(newId) {
        $("#" + newSaveSlotId).attr("id", newId);
        newSaveSlotId = newId;
    }

    /**
     * Inserts the passed HTML-save after the auto save slot.
     * @param {string} save
     * @returns {undefined}
     */
    function insertAfterAutoSaveSlot(save) {
        $("#" + autoSaveSlotId).after(save);
    }

    /**
     * Builds a graphical save slot from the passed meta and inserts it in the
     * correct place in the save list.
     * @param {SaveMetaData} meta The metadata of the new save.
     */
    function buildAndInsertNewSave(meta) {
        var newSave = buildSaveSlot(meta);
        var saves = $("." + gen.saveSlotClass);
        if (saves.length == 0 || $(saves[0]).attr('id') > meta.slot) {
            insertAfterAutoSaveSlot(newSave);
        } else {
            var after = $(saves[0]);
            for (var i = 0; i < saves.length; i++) {
                if (+$(saves[i]).attr('id') < meta.slot) {
                    after = $(saves[i]);
                }
            }
            after.after(newSave);
        }
        var element = $("#" + meta.slot);
        element.hide(0);
        registerListListeners("#" + meta.slot);
        scrollToAndShowNewSave(element);
    }

    /**
     * Scrolls to the newly created save and then shows it.
     * @param {DOM element} element The save.
     */
    function scrollToAndShowNewSave(element) {
        var main = $("#main");
        var currentScroll = main.scrollTop();
        var offset = element.offset().top;

        // Element visible: do nothing.
        if (offset > 75 && offset < 720) {
            element.show(400);
            return;
        }

        var newScroll = currentScroll + offset - 300;
        main.animate({scrollTop: newScroll}, 400, function () {
            element.show(400);
        });
    }

    /**
     * Saves the game, creates and inserts a new save into the UI
     * and updates the save limit, hiding the new save bar if need be.
     * @param {string} comment The textbox-entered comment.
     */
    function newSave(comment) {
        var slot = selected;
        global.saveManager.save(slot, comment);
        var newId = global.saveManager.getFirstOpenSaveSlot();
        if (newId < 0) {
            hideNewSaveSlotAndDisableStuff();
        } else {
            updateNewSaveSlot(newId);
            selected = newId;
        }
        buildAndInsertNewSave(global.saveManager.getMeta(slot));
        gen.updateMaxSaveIndicator();
    }

    /**
     * Rebuilds the slot with the passed id, replacing its contents with the new metadata.
     * @param {string} id The id of the element.
     * @param {SaveMetaData} newMeta The new metadata object.
     */
    function rebuildSlot(id, newMeta) {
        var parent = $("#" + id);
        // .savegender, .savetimestamp, .savename, .savescene, .savecomment
        parent.children().not(".saverank").fadeOut(400, function () {
            var src = GenderEnum.properties[newMeta.gender].icon;
            var element = parent.children(".savegender");
            element.attr("src", src);
            element = parent.children(".savetimestamp");
            element.html(gen.timeToTimePassed(newMeta.time));
            element = parent.children(".savename");
            element.html('<span class="bold">' + newMeta.name + '</span>');
            element = parent.children(".savescene");
            element.html(newMeta.sceneName);
            element = parent.children(".savecomment");
            element.val(newMeta.comment);
            parent.children().fadeIn(400);
        });
    }

    /**
     * Overwrites the selected save after confirming the user's actions.
     * @param {string} comment The textbox-entered comment.
     */
    function overwriteSave(comment) {
        if (!confirm("Do you want to overwrite this save?")) {
            return;
        }
        var oldMeta = global.saveManager.getMeta(selected);
        if (!comment || comment === "") {
            comment = oldMeta.comment;
        }
        global.saveManager.save(selected, comment);
        var newMeta = global.saveManager.getMeta(selected);
        rebuildSlot(selected, newMeta);
    }

    /**
     * Registers a listener for the save button.
     */
    function registerSaveButtonListener() {
        $("#" + saveButtonId).click(function () {
            var comment = $("#" + newCommentFieldId).val();
            if (selected == newSaveSlotId) {
                newSave(comment);
            } else {
                overwriteSave(comment);
            }
        });
    }

    /**
     * Registers listeners to all save slots.
     */
    function registerSaveListeners() {
        registerNewSaveListener();
        registerAutoSaveListener();
        registerListListeners("." + gen.saveSlotClass);
        registerDeleteButtonListener();
        registerSaveButtonListener();
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes this tab.
     * Registers the tab click listener.
     * Registers the tab with the scene loader.
     */
    this.init = function () {
        registerListeners();
        global.sceneLoader.registerScene(id, this);
    };

    /**
     * Loads this tab.
     */
    this.load = function () {
        gen.load();
        newSaveSlotId = "";
        var tab = buildTab();
        global.ui.loadToMain(tab);
        registerSaveListeners();
    };
}
